#include "options.h"
#include "ui_options.h"
#include "memorycardselect.h"
#include "accelerometerconfig.h"

QStringList current_config=QStringList();
QString current_core="";
options::options(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::options)
{
    ui->setupUi(this);
#ifdef Q_WS_MAEMO_5
    this->setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
    this->setWindowFlags(this->windowFlags() | Qt::Window);
    connect(ui->checkBox_2,SIGNAL(clicked()),this,SLOT(psx4m_hwa_block()));
    connect(ui->checkBox_9,SIGNAL(toggled(bool)),this,SLOT(accel_toggled(bool)));
    connect(ui->pbnMemoryCard1, SIGNAL(clicked()),this,SLOT(memory_card_1()));
    connect(ui->pbnMemoryCard2, SIGNAL(clicked()),this,SLOT(memory_card_2()));
    connect(ui->pbnConfigAccel, SIGNAL(clicked()), this, SLOT(config_accel()));

    QDir dir( bios_path);
    dir.setFilter(QDir::Files);
    dir.setSorting(QDir::Name);
    dir.setNameFilters(QStringList()<<"*.bin");
    QFileInfoList list = dir.entryInfoList();
    for (int i = 0; i < list.size(); ++i) {
        QFileInfo fileInfo = list.at(i);
        ui->comboBox_4->addItem(fileInfo.fileName());
    }
    load_cfg();

    m_Loop = NULL;
}

options::~options()
{
    delete ui;

    if (m_Loop)
        delete m_Loop;
}
void options::hideEvent(QHideEvent *){
    if (!this->isVisible()){
        m_Loop->quit();

        QDialog *dlg=new QDialog(this);
        dlg->setWindowTitle(tr("Confirm"));
        QGridLayout *lay=new QGridLayout(dlg);
        QLabel *lbl=new QLabel(tr("Do you want to save settings?"),dlg);
        lay->addWidget(lbl,1,0,1,2);
        QPushButton *yes=new QPushButton(tr("Yes"),dlg);
        QPushButton *no=new QPushButton(tr("No"),dlg);
        connect (yes,SIGNAL(clicked()),dlg,SLOT(accept()));
        connect (no,SIGNAL(clicked()),dlg,SLOT(reject()));
        connect (dlg,SIGNAL(accepted()),this,SLOT(save_cfg()));
        connect (dlg,SIGNAL(rejected()),this,SLOT(reload_cfg()));
        lay->addWidget(yes,2,0);
        lay->addWidget(no ,2,1);
        dlg->exec();
    }
}

void options::load_cfg(){

    m_MemoryCard1 = "/home/user/.pcsx/memcards/card1.mcd";
    m_MemoryCard2 = "/home/user/.pcsx/memcards/card2.mcd";

    QFileInfo f(m_MemoryCard1);
    ui->pbnMemoryCard1->setText(f.fileName());
    f = QFileInfo(m_MemoryCard2);
    ui->pbnMemoryCard2->setText(f.fileName());

    QString path="";
    if (game!="")
       path=cfg_path+game;
    else
       path=cfg_path+"default";

    QStringList configs = QStringList() << "core" << "sound" << "bios";
    configs << "psx4m-cycle" << "psx4m-frame" << "psx4m-hwa" << "psx4m-fps" << "psx4m-fullscreen" << "psx4m-afix";
    configs << "pcsx-threadedspu" << "pcsx-hwa" << "pcsx-bios" << "pcsx-frameskip";
    configs << "accel" << "cdda" << "xa" << "rcnt" << "sio" << "spuirq" << "vsync" << "fsaa" << "pcsx-fullscreen";
    configs << "pcsx-fps" << "pcsx-cpu" << "pcsx-nofl" << "mcd1" << "mcd2" << "spu_reverb" << "spu_interpolation";
    configs << "region" << "analog" << "accel_sens" << "accel_y_def" << "accel_max" << "displayon";

    ui->comboBox_5->setCurrentIndex(0);
    ui->checkBox_9->setChecked(false);
    ui->checkBox_4->setChecked(false);
    ui->checkBox_18->setChecked(false);
    ui->checkBox_19->setChecked(false);
    ui->chkReverb->setChecked(false);
    ui->cmbInterpolation->setCurrentIndex(0);
    ui->cmbRegion->setCurrentIndex(0);

    ui->chkDisplayOn->setChecked(false);
    ui->numSens->setValue(150);
    ui->numYDef->setValue(500);
    ui->numMaxValue->setValue(500);

    if (QFile::exists(path)){
        QFile file(path);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)){
            qDebug(QString("Can't open file : %1\n").arg(path).toAscii());
            return;
        }

             QTextStream in(&file);
             while (!in.atEnd()) {
                 QStringList line = QString(in.readLine()).split("=");
                 QString value=line.at(1);
                 switch (configs.indexOf(line.at(0).trimmed())){
                 case 0: // core
                     if (value=="1")ui->radioButton_2->setChecked(true); else ui->radioButton->setChecked(true);
                     break;
                 case 1: // sound
                     if (value=="1")ui->checkBox->setChecked(true);else ui->checkBox->setChecked(false);
                     break;
                 case 2: // bios
                     if (value=="")value="scph1001.bin";
                     if (ui->comboBox_4->findText(value)!=1)
                         ui->comboBox_4->setCurrentIndex(ui->comboBox_4->findText(value));
                     break;
                 case 3: // psx4m-cycle
                     if (ui->comboBox->findText(value)!=1)
                         ui->comboBox->setCurrentIndex(ui->comboBox->findText(value));
                     break;
                 case 4: // psx4m-frame
                     if (ui->comboBox_2->findText(value)!=1)
                         ui->comboBox_2->setCurrentIndex(ui->comboBox_2->findText(value));
                     break;
                 case 5: // psx4m-hwa
                     if (value=="1")ui->checkBox_2->setChecked(true);else ui->checkBox_2->setChecked(false);
                     break;
                 case 6: //psx4m-fps
                     if (value=="1")ui->checkBox_6->setChecked(true);else ui->checkBox_6->setChecked(false);
                     break;
                 case 7: //psx4m-fullscreen
                     if (value=="1")ui->checkBox_7->setChecked(true);else ui->checkBox_7->setChecked(false);
                     break;
                 case 8: //psx4m-afix
                     if (value=="1")ui->checkBox_8->setChecked(true);else ui->checkBox_8->setChecked(false);
                     break;
                 case 9: // pcsx-threadedspu
                      if (value=="1")ui->checkBox_3->setChecked(true);else ui->checkBox_3->setChecked(false);
                      break;
                 case 10:// pcsx-hwa
                     if (ui->comboBox_5->findText(value)!=-1)
                         ui->comboBox_5->setCurrentIndex(ui->comboBox_5->findText(value));
                      break;
                 case 11:// pcsx-bios
                      if (value=="1")ui->checkBox_5->setChecked(true);else ui->checkBox_5->setChecked(false);                      
                      break;
                 case 12:// pcsx-frameskip
                      if (ui->comboBox_3->findText(value)!=-1)
                          ui->comboBox_3->setCurrentIndex(ui->comboBox_3->findText(value));
                      break;
                 case 13:
                      if (value=="1")ui->checkBox_9->setChecked(true);else ui->checkBox_9->setChecked(false);                      
                      break;
                 case 14:
                      if (value=="1")ui->checkBox_10->setChecked(true);else ui->checkBox_10->setChecked(false);
                      break;
                 case 15:
                      if (value=="1")ui->checkBox_11->setChecked(true);else ui->checkBox_11->setChecked(false);
                      break;
                 case 16:
                      if (value=="1")ui->checkBox_12->setChecked(true);else ui->checkBox_12->setChecked(false);
                      break;
                 case 17:
                      if (value=="1")ui->checkBox_13->setChecked(true);else ui->checkBox_13->setChecked(false);
                      break;
                 case 18:
                      if (value=="1")ui->checkBox_14->setChecked(true);else ui->checkBox_14->setChecked(false);
                      break;
                 case 19:
                      if (value=="1")ui->checkBox_15->setChecked(true);else ui->checkBox_15->setChecked(false);
                      break;
                 case 20:
                      if (value=="1")ui->checkBox_16->setChecked(true);else ui->checkBox_16->setChecked(false);
                      break;
                 case 21:
                      if (value=="1")ui->checkBox_17->setChecked(true);else ui->checkBox_17->setChecked(false);
                      break;
                 case 22:
                      if (value=="1")ui->checkBox_4->setChecked(true);else ui->checkBox_4->setChecked(false);
                      break;
                 case 23:
                      if (value=="1")ui->checkBox_18->setChecked(true);else ui->checkBox_18->setChecked(false);
                      break;
                 case 24:
                      if (value=="1")ui->checkBox_19->setChecked(true);else ui->checkBox_19->setChecked(false);
                      break;
                 case 25:{
                         m_MemoryCard1 = value;
                         f = QFileInfo(m_MemoryCard1);
                         ui->pbnMemoryCard1->setText(f.fileName());
                      }
                      break;
                 case 26:{
                         m_MemoryCard2 = value;
                         f = QFileInfo(m_MemoryCard2);
                         ui->pbnMemoryCard2->setText(f.fileName());
                      }
                      break;
                 case 27:
                      if (value=="1")ui->chkReverb->setChecked(true);else ui->chkReverb->setChecked(false);
                      break;
                 case 28:
                     if (ui->cmbInterpolation->findText(value)!=1)
                         ui->cmbInterpolation->setCurrentIndex(ui->cmbInterpolation->findText(value));
                     break;
                 case 29:
                     if (ui->cmbRegion->findText(value)!=1)
                         ui->cmbRegion->setCurrentIndex(ui->cmbRegion->findText(value));
                     break;
                 case 30:
                     if (value=="1")ui->chkAnalog->setChecked(true);else ui->chkAnalog->setChecked(false);
                     break;
                 case 31:
                     ui->numSens->setValue(value.toInt());
                     break;
                 case 32:
                     ui->numYDef->setValue(value.toInt());
                     break;
                 case 33:
                     ui->numMaxValue->setValue(value.toInt());
                     break;
                 case 34:
                     if (value=="1")ui->chkDisplayOn->setChecked(true);else ui->chkDisplayOn->setChecked(false);
                     break;
                 default:
                      qDebug(QString("Undefined setting:%1 = %2").arg(line.at(0)).arg(value).toAscii());
                      break;
                 }
             }
             file.close();

             // checking
             psx4m_hwa_block();
             accel_toggled(ui->checkBox_9->isChecked());

             if (!ui->comboBox_4->currentIndex()>-1)ui->comboBox_4->setCurrentIndex(0);

             createCfg();
    }
}


void options::createCfg(){
    current_config=QStringList();
    if (ui->checkBox->isChecked())current_config<<"-nosound";
    if (ui->checkBox_9->isChecked())current_config<<"-accel";

    if (ui->checkBox_10->isChecked())current_config<<"-xa";
    if (ui->checkBox_11->isChecked())current_config<<"-cdda";
    if (ui->checkBox_12->isChecked())current_config<<"-sio";
    if (ui->checkBox_13->isChecked())current_config<<"-spuirq";
    if (ui->checkBox_14->isChecked())current_config<<"-rcnt";
    if (ui->checkBox_15->isChecked())current_config<<"-vsync";    
    current_config<<"-mcd1"<<m_MemoryCard1;
    current_config<<"-mcd2"<<m_MemoryCard2;

    if (ui->radioButton_2->isChecked())
    {
        current_core="pcsx";

        if (ui->chkDisplayOn->isChecked())current_config<<"-displayon";
        if (ui->chkAnalog->isChecked())current_config<<"-analog";
        current_config<<"-sens"<<QString::number(ui->numSens->value());
        current_config<<"-ydef"<<QString::number(ui->numYDef->value());
        current_config<<"-max"<<QString::number(ui->numMaxValue->value());

        if (ui->checkBox_3->isChecked())current_config<<"-sputhreaded";
        if (ui->comboBox_5->currentText()=="oldgpu")current_config<<"-oldgpu";
        else if (ui->comboBox_5->currentText()=="gpu-gles")current_config<<"-gles";
        else if (ui->comboBox_5->currentText()=="gpu_unai")current_config<<"-unai";
        else if (ui->comboBox_5->currentText()=="gpu_4all")current_config<<"-4all";

        if (ui->checkBox_5->isChecked())
            if (!ui->comboBox_4->currentText().isEmpty() && !bios_path.isEmpty())
                      current_config<<"-bios"<<ui->comboBox_4->currentText()<<"-bdir"<<bios_path;
        if (ui->comboBox_3->currentIndex()>=0)
            current_config<<"-frameskip"<<QString("%1").arg(ui->comboBox_3->currentIndex() - 1);
        if (ui->checkBox_17->isChecked())current_config<<"-fullscreen";
        if (ui->checkBox_4->isChecked())current_config<<"-fps";
        if (ui->checkBox_18->isChecked())current_config<<"-cpu";
        if (ui->checkBox_19->isChecked())current_config<<"-nofl";

        if (ui->chkReverb->isChecked())current_config<<"-spu_reverb"<<"1";
        if (ui->cmbInterpolation->currentIndex()>=0)
            current_config<<"-spu_interpolation"<<QString("%1").arg(ui->cmbInterpolation->currentIndex());
        if (ui->cmbRegion->currentIndex()>=0)
            current_config<<"-region"<<QString("%1").arg(ui->cmbRegion->currentIndex() - 1);

        current_config<<"-cdfile";

    }
    else{
        if (ui->comboBox->currentText()!="Disabled")current_config<<"-cyclemult"<<ui->comboBox->currentText();
        current_config<<"-bios"<<ui->comboBox_4->currentText()<<"-bdir"<<bios_path;
    if (ui->checkBox_2->isChecked())
        {
        current_core="psx4m-gles";

        }
    else
    {
        current_core="psx4m";
        if (ui->comboBox_2->currentText()!="Disabled"){
            if (ui->comboBox_2->currentText()=="Auto")current_config<<"-autoskip";
            else current_config<<"-frameskip"<<ui->comboBox_2->currentText();
        }
        if (ui->checkBox_6->isChecked())current_config<<"-showfps";
        if (ui->checkBox_7->isChecked())current_config<<"-fullscreen";
        if (ui->checkBox_8->isChecked())current_config<<"-abbeyfix";



              }
    current_config<<"-runcd";
}
}

void options::reload_cfg()
{
    load_cfg();
}

void options::save_cfg(){
    createCfg();
    QString path="";
    if (game!="")
        path=cfg_path+game;
    else
       path=cfg_path+"default";

    QString config=QString(
                   "core=%1\n" \
                   "sound=%2\n" \
                   "bios=%3\n" \
                   "psx4m-cycle=%4\n" \
                   "psx4m-frame=%5\n" \
                   "psx4m-hwa=%6\n" \
                   "psx4m-fps=%7\n" \
                   "psx4m-fullscreen=%8\n" \
                   "psx4m-afix=%9\n" \
                   "pcsx-threadedspu=%10\n" \
                   "pcsx-hwa=%11\n" \
                   "pcsx-bios=%12\n" \
                   "pcsx-frameskip=%13\n" \
                   "accel=%14\n" \
                   "cdda=%15\n" \
                   "xa=%16\n" \
                   "rcnt=%17\n" \
                   "sio=%18\n" \
                   "spuirq=%19\n" \
                   "vsync=%20\n" \
                   "fsaa=%21\n" \
                   "pcsx-fullscreen=%22\n"\
                   "pcsx-fps=%23\n" \
                   "pcsx-cpu=%24\n" \
                   "pcsx-nofl=%25\n" \
                   "mcd1=%26\n" \
                   "mcd2=%27\n" \
                   "spu_reverb=%28\n" \
                   "spu_interpolation=%29\n" \
                   "region=%30\n" \
                   "analog=%31\n" \
                   "accel_sens=%32\n" \
                   "accel_y_def=%33\n" \
                   "accel_max=%34\n" \
                   "displayon=%35\n")
            .arg(ui->radioButton_2->isChecked()?"1":"0")
            .arg(ui->checkBox->isChecked()?"1":"0")
            .arg(ui->comboBox_4->currentText())
            .arg(ui->comboBox->currentText())
            .arg(ui->comboBox_2->currentText())
            .arg(ui->checkBox_2->isChecked()?"1":"0")
            .arg(ui->checkBox_6->isChecked()?"1":"0")
            .arg(ui->checkBox_7->isChecked()?"1":"0")
            .arg(ui->checkBox_8->isChecked()?"1":"0")
            .arg(ui->checkBox_3->isChecked()?"1":"0")
            .arg(ui->comboBox_5->currentText())
            .arg(ui->checkBox_5->isChecked()?"1":"0")
            .arg(ui->comboBox_3->currentText())
            .arg(ui->checkBox_9->isChecked()?"1":"0")
            .arg(ui->checkBox_10->isChecked()?"1":"0")
            .arg(ui->checkBox_11->isChecked()?"1":"0")
            .arg(ui->checkBox_12->isChecked()?"1":"0")
            .arg(ui->checkBox_13->isChecked()?"1":"0")
            .arg(ui->checkBox_14->isChecked()?"1":"0")
            .arg(ui->checkBox_15->isChecked()?"1":"0")
            .arg(ui->checkBox_16->isChecked()?"1":"0")
            .arg(ui->checkBox_17->isChecked()?"1":"0")
            .arg(ui->checkBox_4->isChecked()?"1":"0")
            .arg(ui->checkBox_18->isChecked()?"1":"0")
            .arg(ui->checkBox_19->isChecked()?"1":"0")
            .arg(m_MemoryCard1)
            .arg(m_MemoryCard2)
            .arg(ui->chkReverb->isChecked()?"1":"0")
            .arg(ui->cmbInterpolation->currentText())
            .arg(ui->cmbRegion->currentText())
            .arg(ui->chkAnalog->isChecked()?"1":"0")
            .arg(ui->numSens->value())
            .arg(ui->numYDef->value())
            .arg(ui->numMaxValue->value())
            .arg(ui->chkDisplayOn->isChecked()?"1":"0");

    QFile file(path);
    QFileInfo pathInfo(path);
    if (!QDir().exists(pathInfo.path()))
        QDir().mkpath(pathInfo.path());

    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
        qDebug(QString("Can't save %1").arg(path).toAscii());
        return;
    }

    QTextStream out(&file);
    out <<config;

    file.close();
}

void options::psx4m_hwa_block(){
    if (ui->checkBox_2->checkState()==Qt::Checked){
        ui->checkBox_6->setEnabled(false);
        ui->checkBox_7->setEnabled(false);
        ui->checkBox_8->setEnabled(false);
        ui->comboBox_2->setEnabled(false);
        ui->checkBox_16->setEnabled(true);
    }else{
        ui->checkBox_6->setEnabled(true);
        ui->checkBox_7->setEnabled(true);
        ui->checkBox_8->setEnabled(true);
        ui->comboBox_2->setEnabled(true);
        ui->checkBox_16->setEnabled(false);
    }
}

void options::memory_card_1()
{
#ifdef Q_WS_MAEMO_5
    QString filename = GetMemoryCardName(m_MemoryCard1);
#else
    QString filename = QFileDialog::getOpenFileName(this, tr("Open Memory card"),
                                                    QDir::homePath(),
                                                    tr("Memory cards (*.mcd)"));
#endif
    if (!filename.isEmpty()){
        m_MemoryCard1 = filename;
        QFileInfo f(m_MemoryCard1);
        ui->pbnMemoryCard1->setText(f.fileName());
    }
}

void options::memory_card_2()
{
#ifdef Q_WS_MAEMO_5
    QString filename = GetMemoryCardName(m_MemoryCard2);
#else
    QString filename = QFileDialog::getOpenFileName(this, tr("Open Memory card"),
                                                    QDir::homePath(),
                                                    tr("Memory cards (*.mcd)"));
#endif
    if (!filename.isEmpty()){
        m_MemoryCard2 = filename;
        QFileInfo f(m_MemoryCard2);
        ui->pbnMemoryCard2->setText(f.fileName());
    }
}

#ifdef Q_WS_MAEMO_5
QString options::GetMemoryCardName(QString selected)
{
    MemoryCardSelect* dlg = new MemoryCardSelect(this, selected);
    if (dlg->exec() == QDialog::Accepted)
        return dlg->Selected();
    return "";
}
#endif

void options::show()
{
    ui->tabWidget->setCurrentIndex(0);

    ui->scrollArea->verticalScrollBar()->setValue(0);
    ui->scrollArea_2->verticalScrollBar()->setValue(0);
    ui->scrollArea_3->verticalScrollBar()->setValue(0);
    ui->scrollArea_4->verticalScrollBar()->setValue(0);

    QWidget::show();

    m_Loop = new QEventLoop(this);
    m_Loop->exec();

}

void options::accel_toggled(bool toggled)
{
    ui->chkAnalog->setEnabled(toggled);
    ui->numSens->setEnabled(toggled);
    ui->numYDef->setEnabled(toggled);
    ui->numMaxValue->setEnabled(toggled);
    ui->pbnConfigAccel->setEnabled(toggled);
}

void options::config_accel()
{
    AccelerometerConfig* dlg = new AccelerometerConfig(this, ui->numSens->value(), ui->numYDef->value(), ui->numMaxValue->value());
    dlg->show();

    ui->numSens->setValue(dlg->GetSens());
    ui->numYDef->setValue(dlg->GetYDef());
    ui->numMaxValue->setValue(dlg->GetMaxValue());

    delete dlg;
}
